@@ -147,7 +147,9 @@ connect' host serv = do
                              , addrProtocol = proto
                              , addrSocketType = Stream }
     addrs <- getAddrInfo (Just hints) (Just host) (Just serv)
-    let addr = head addrs
+    firstSuccessful $ map tryToConnect addrs
+  where
+  tryToConnect addr =
     bracketOnError
 	(socket (addrFamily addr) (addrSocketType addr) (addrProtocol addr))
 	(sClose)  -- only done if there's an error
@@ -158,13 +160,16 @@ connect' host serv = do
 #endif
 
 -- | Creates the server side socket which has been bound to the
--- specified port.  
+-- specified port.
 --
 -- NOTE: To avoid the \"Address already in use\"
 -- problems popped up several times on the GHC-Users mailing list we
 -- set the 'ReuseAddr' socket option on the listening socket.  If you
 -- don't want this behaviour, please use the lower level
 -- 'Network.Socket.listen' instead.
+--
+-- If available, the 'IPv6Only' socket option is set to 0
+-- so that both IPv4 and IPv6 can be accepted with this socket.
 
 listenOn :: PortID 	-- ^ Port Identifier
 	 -> IO Socket	-- ^ Connected Socket
@@ -222,11 +227,17 @@ listen' :: ServiceName -> IO Socket
 
 listen' serv = do
     proto <- getProtocolNumber "tcp"
+    -- We should probably specify addrFamily = AF_INET6 and the filter
+    -- code below should be removed. AI_ADDRCONFIG is probably not
+    -- necessary. But this code is well-tested. So, let's keep it.
     let hints = defaultHints { addrFlags = [AI_ADDRCONFIG, AI_PASSIVE]
                              , addrSocketType = Stream
                              , addrProtocol = proto }
     addrs <- getAddrInfo (Just hints) Nothing (Just serv)
-    let addr = head addrs
+    -- Choose an IPv6 socket if exists.  This ensures the socket can
+    -- handle both IPv4 and IPv6 if v6only is false.
+    let addrs' = filter (\x -> addrFamily x == AF_INET6) addrs
+        addr = if null addrs' then head addrs else head addrs'
     bracketOnError
         (socket (addrFamily addr) (addrSocketType addr) (addrProtocol addr))
 	(sClose)
@@ -448,3 +459,12 @@ catchIO = Exception.catch
 catchIO = Exception.catchJust Exception.ioErrors
 #endif
 
+-- Returns the first action from a list which does not throw an exception.
+-- If all the actions throw exceptions (and the list of actions is not empty),
+-- the last exception is thrown.
+firstSuccessful :: [IO a] -> IO a
+firstSuccessful [] = error "firstSuccessful: empty list"
+firstSuccessful (p:ps) = catchIO p $ \e ->
+    case ps of
+        [] -> Exception.throw e
+        _  -> firstSuccessful ps
