@@ -1,4 +1,4 @@
-{-# OPTIONS_GHC -fglasgow-exts -cpp #-}
+{-# LANGUAGE CPP #-}
 --------------------------------------------------------------------------------
 -- |
 --  Module      :  Network.URI
@@ -125,11 +125,11 @@ import Control.Monad( MonadPlus(..) )
 
 #ifdef __GLASGOW_HASKELL__
 import Data.Typeable  ( Typeable )
-#ifdef BASE4
+# if MIN_VERSION_base(4,0,0)
 import Data.Data      ( Data )
-#else
+# else
 import Data.Generics  ( Data )
-#endif
+# endif
 #else
 import Data.Typeable  ( Typeable(..), TyCon, mkTyCon, mkTyConApp )
 #endif
@@ -1070,6 +1070,7 @@ relativeFrom uabs base
         , uriQuery     = ""
         }
     where
+        diff :: Eq b => (a -> b) -> a -> a -> Bool
         diff sel u1 u2 = sel u1 /= sel u2
         -- Remove dot segments except the final segment
         removeBodyDotSegments p = removeDotSegments p1 ++ p2
